from typing import Literal
from fastapi import HTTPException
from fastapi.param_functions import Optional, Cookie
from jose import JWTError, jwt
from pydantic import BaseModel
from starlette import status
import hashlib
from loguru import logger
from base64 import b64decode
from json import loads
from datetime import datetime, timedelta

from jwtserver.settings import TokenModel


class Data(BaseModel):
    access_token: Optional[str] = None
    refresh_token: Optional[str] = None


def secret(string, base_sol: str, temporary_sol: str = ""):
    sol = f"{base_sol[:50]}{temporary_sol}){base_sol[:50]}"
    return hashlib.pbkdf2_hmac(
        'sha256',
        string.encode('ascii'),
        sol.encode('ascii'),
        100000
    ).hex()


class TokenProcessor:
    def __init__(
            self,
            _config: TokenModel,
            refresh_token: str = None,
            access_token: str = None,
    ):
        self.config = _config
        self.access = access_token
        self.new_access = access_token
        self.refresh = refresh_token
        self.new_refresh = refresh_token

    def payload_token_untested(self, token_type: Literal['access', 'refresh']):
        """Token untested payload
        :param str token_type:
        :return dict data: token payload
        """
        return loads(
            b64decode(getattr(self, token_type).split('.', 2)[1] + '=='))

    def payload_token(self, token_type: Literal['access', 'refresh']):
        """Token tested payload
        :param str token_type:
        :return dict data: token payload
        :raises JWTError: If the signature is invalid in any way
        :raises ExpiredSignatureError: If the signature has expired
        :raises JWTClaimsError: If any claim is invalid in any way
        """
        try:
            return jwt.decode(
                getattr(self, token_type), self.config.secret_key,
                algorithms=[self.config.algorithm])
        except JWTError:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Bad JWT token",
                headers={"WWW-Authenticate": "JSv1"})

    def create_pair_tokens(self, uuid):
        """Create two JWT tokens with a shared secret at the same time
        :return tuple: [access_token, refresh_token]
        """
        access_time = timedelta(minutes=self.config.access_expire_time)
        refresh_time = timedelta(minutes=self.config.refresh_expire_time)

        datetime_now = datetime.now()
        temporary_sol_access = (datetime_now + access_time).timestamp()
        temporary_sol_refresh = (datetime_now + refresh_time).timestamp()
        payload_access = {
            'uuid': uuid,
            'secret': secret(
                uuid,
                base_sol=self.config.base_sol,
                temporary_sol=str(temporary_sol_access))[:32],
            'exp': temporary_sol_access
        }

        payload_refresh = {
            'secret': secret(
                uuid,
                base_sol=self.config.base_sol,
                temporary_sol=str(temporary_sol_refresh))[32:],
            'exp': temporary_sol_refresh,
        }

        access_jwt = jwt.encode(
            payload_access,
            self.config.secret_key,
            algorithm=self.config.algorithm)

        refresh_jwt = jwt.encode(
            payload_refresh,
            self.config.secret_key,
            algorithm=self.config.algorithm)

        logger.info(f'create new {access_jwt=}')
        logger.info(f'create mew {refresh_jwt=}')
        return access_jwt, refresh_jwt



from multiprocessing import cpu_count
from jwtserver.functions.config import load_config

config = load_config().server


def max_workers():
    return cpu_count()


bind = f'{config.host}:{config.port}'
max_requests = 1000
worker_class = 'uvicorn.workers.UvicornWorker'
workers = max_workers()
threads = max_workers() * 2
reload = True
name = 'troll'



import httpx
from random import randint
from pydantic import BaseModel, PyObject

__all__ = ['SMSC']


def string_to_dict(text: str):
    return {
        k.strip(): v.strip()
        for k, v in [b.split(' - ', 1) for b in text.split(',')]
    }


class SMSModel(BaseModel):
    debug: bool = True  # imitation of sending SMS messages.
    ignore_attempts: bool = False
    try_call: int = 2
    try_sms: int = 2
    block_time_minutes: int = 180
    sms_fabric: PyObject = 'jwtserver.internal.SMSC.SMSC'
    login: str = None
    password: str = None
    time_sms: int = 120
    time_call: int = 90


class SMSC:
    url = 'https://smsc.ru/sys/send.php?'

    def __init__(self, _config):
        self.config: SMSModel = _config
        self.param_url = {
            'login': self.config.login,
            'psw': self.config.password
        }

    async def calling(self, telephone) -> tuple[str | None, str | None]:
        param_url = {
            **self.param_url,
            **{'phones': telephone, 'mes': 'code', 'call': 1}}

        if self.config.debug:
            code = '{:0>4d}'.format(randint(0, 9999))
        else:
            async with httpx.AsyncClient() as client:
                resp: httpx.Response = await client.get(self.url, param_url)
                if resp.status_code == httpx.codes.OK:
                    text = resp.text
                    if "ERROR" in text:
                        return None, 'Wait1min'
                    data = string_to_dict(text)
                else:
                    return None, 'response status not 200 OK'

            code = data['CODE'][-4:]
        return code, None

    async def send_sms(self, telephone, code) -> [bool, str | None]:
        param_url = {
            **self.param_url,
            **{'phones': telephone, 'mes': code}
        }
        if self.config.debug:
            return True, None

        async with httpx.AsyncClient() as client:
            resp: httpx.Response = await client.get(self.url, param_url)
            if resp.status_code == httpx.codes.OK:
                text = resp.text
                if "ERROR" in text:
                    return False, 'Wait1min'
            else:
                return False, 'response status not 200 OK'
            return True, None



import datetime
from secrets import token_hex
from aioredis import Redis
from fastapi import Depends, Body, APIRouter, HTTPException, Response
from loguru import logger
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.future import select
from starlette import status
from jwtserver.internal.SMSCRules import SMSRules, SendDetailsModel, \
    SendCodeReturnModel
from jwtserver.settings import Settings, get_settings
from jwtserver.Google.Recaptcha_v3 import Recaptcha
from jwtserver.api.v1.help_func.ParseToken import TokenProcessor
from jwtserver.dependencies.init_redis import redis_conn
from jwtserver.dependencies.session_db import async_db_session
from jwtserver.functions.secure import get_password_hash
from jwtserver.models import User
from pydantic import BaseModel

router = APIRouter(prefix='/api/v1', tags=['Registration'])


class ResponsePhoneStatusModel(BaseModel):
    free: bool
    telephone: str


class CheckCodeResponseModel(BaseModel):
    reg_token: str


class RespError(BaseModel):
    error: str
    block_time: int | None


@router.post('/phone_status', response_model=ResponsePhoneStatusModel,
             name='reg:phone_status')
async def phone_status(
        telephone: str = Body(...),
        recaptcha_token: str = Body(...),
        session: AsyncSession = Depends(async_db_session),
        settings: Settings = Depends(get_settings),
):
    recaptcha = Recaptcha(
        _config=settings.Google.Recaptcha,
        environment=settings.environment,
        recaptcha_token=recaptcha_token,
    )
    await recaptcha.set_action_name('SignUpPage/PhoneStatus').greenlight()
    stmt = select(User).where(User.telephone == telephone)
    result = await session.execute(stmt)
    busy = result.scalars().first()
    return {'free': False if busy else True, 'telephone': telephone}


class RespSendCodeModel(BaseModel):
    send: bool
    block_time: float
    method: str


@router.post('/send_code', response_model=RespSendCodeModel,
             description='Sending a code through a call or SMS',
             name='reg:send_code')
async def send_code(
        telephone: str = Body(...),
        settings: Settings = Depends(get_settings),
        redis: Redis = Depends(redis_conn),
):
    sms_rules = SMSRules(settings, redis)
    result = await sms_rules.send_code(telephone)

    if result.error:
        raise HTTPException(
            status_code=status.HTTP_409_CONFLICT,
            detail={
                "error": result.error,
                'block_time': result.error.details.block_time
            }
        )

    return {
        "send": True,
        "block_time": result.is_sent.block_time.timestamp(),
        "method": result.is_sent.method,
    }


@router.post('/check_code',
             description="User authorization by login and password",
             # response_description=response_description,
             response_model=CheckCodeResponseModel,

             )
async def check_code(
        # redis: Redis,
        telephone: str = Body(...),
        code: int = Body(...),
        redis: Redis = Depends(redis_conn),
        recaptcha: Recaptcha = Depends(Recaptcha)
):
    """Checking the code from SMS or Call
    :param str telephone: Telephone number in international format
    :param int code: 4 digit verification code
    :param redis: Redis client
    :param recaptcha: Validate Google recaptcha_v3.md v3 [return True or HTTPException]
    :return: one-time token for registration
    """
    await recaptcha.set_action_name('SignUpPage/CheckCode').greenlight()

    code_method = await redis.get(telephone)
    if code_method:
        from_redis_code, method = code_method.split(":")
        if int(from_redis_code) == code:
            reg_token = token_hex(16)
            await redis.set(f"{telephone}_reg_token", reg_token, 60 * 60)
            return {"reg_token": reg_token}

        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Неверный код",
        )
    raise HTTPException(
        status_code=status.HTTP_400_BAD_REQUEST,
        detail="Нужно запросить новый код ",
    )


class Data(BaseModel):
    telephone: str
    password: str
    reg_token: str


class AccessTokenResponseModel(BaseModel):
    access_token: str
    token_type: str


@router.post("/signup",
             response_model=AccessTokenResponseModel,
             description="Registration user by login and password",
             status_code=status.HTTP_201_CREATED)
async def reg_user(
        # redis: Redis,
        response: Response,
        data: Data = None,
        # form_data: RegRequestForm = Depends(),
        redis: Redis = Depends(redis_conn),
        session: AsyncSession = Depends(async_db_session)
):
    if not await redis.get(f"{data.telephone}_reg_token"):
        return {"error": "bad reg token"}
    else:
        stmt = select(User).where(User.telephone == data.telephone)
        if (await session.execute(stmt)).scalars().first():
            return {"error": "user exist"}
        else:
            hashed_password = get_password_hash(data.password)
            new_user = User(telephone=data.telephone, is_active=True,
                            password=hashed_password)
            session.add(new_user)
            await session.commit()
            token_processor = TokenProcessor()
            access_token, refresh_token = token_processor.create_pair_tokens(
                new_user.uuid.hex)

            await redis.delete(data.telephone)
            await redis.delete(f"{data.telephone}_reg_token")

            response.set_cookie(
                key='refresh_token',
                value=refresh_token,
                httponly=True,
                secure=True,
                max_age=settings.token.refresh_expire_time * 60)
            return {"access_token": access_token, "token_type": "JSv1"}


__all__ = ['router']



from passlib.context import CryptContext

pwd_context = CryptContext(schemes=["sha256_crypt", "ldap_salted_md5"],
                           sha256_crypt__default_rounds=649342,
                           ldap_salted_md5__salt_size=16)


def verify_password(plain_password, hashed_password):
    return pwd_context.verify(plain_password, hashed_password)


def get_password_hash(password):
    return pwd_context.hash(password)



from typing import Optional
from pydantic import BaseModel
from sqlalchemy.ext.asyncio import AsyncSession
from starlette import status

from jwtserver.api.v1.help_func.gen_token_secret import secret
from jwtserver.functions.config import load_config
from jwtserver.models import User
from jwtserver.app import app
from fastapi import Depends, Cookie, HTTPException, Response, Header

from jwtserver.api.v1.help_func.ParseToken import TokenProcessor
from jwtserver.functions.session_db import async_db_session

config_token = load_config().token


class UpdateTokenResponseModel(BaseModel):
    access_token: str


async def get_current_active_user(
        Authorization: str = Header(...),
        session: AsyncSession = Depends(async_db_session)
):
    credentials_exception = HTTPException(
        status_code=status.HTTP_401_UNAUTHORIZED,
        detail="User is inactive",
        headers={"WWW-Authenticate": "JSv1"},
    )
    token = TokenProcessor(access_token=Authorization.split()[1])
    user = await session.get(User, token.payload_token_untested('access')['uuid'])
    if not user.is_active:
        raise credentials_exception
    return user


@app.get("/api/v1/update_token/",
         tags=["Authorization"],
         response_model=UpdateTokenResponseModel,
         description="Update access_token and refresh_token",
         response_description="refresh_token set cookie")
async def update_token(
        response: Response,
        Authorization: str = Header(...),
        refresh_token: Optional[str] = Cookie(None),
        current_active_user: User = Depends(get_current_active_user),
):
    """
    More here https://jwtserver.darkdeal.net/en/api_v1/update-token/
    :param response: Fastapi response
    :param Authorization: 'token_type access_token'
    :param refresh_token: from cookie
    :param current_active_user: Depends on get_current_active_user
    :raises HTTPException: If user is not active
    :return: UpdateTokenResponseModel
    """
    token = TokenProcessor(access_token=Authorization.split()[1], refresh_token=refresh_token)
    payload_access_untested = token.payload_token_untested('access')
    payload_refresh_token = token.payload_token('refresh')

    try:
        _secret_full = payload_access_untested['secret'] + payload_refresh_token['secret']

        if not _secret_full:
            return {"error": "invalid pair of tokens"}

        if _secret_full == secret(payload_access_untested['uuid'], payload_access_untested['exp']):
            access_token, refresh_token = token.create_pair_tokens(current_active_user.uuid.hex)

            response.set_cookie(
                key='refresh_token',
                value=refresh_token,
                httponly=True,
                secure=True,
                max_age=config_token.refresh_expire_time * 60)

            return {"access_token": access_token}
    except AttributeError:
        return {"error": "token won't find"}



import phonenumbers
from fastapi import HTTPException


def telephone_validate(telephone):
    if not phonenumbers.is_valid_number(phonenumbers.parse(telephone, None)):
        raise HTTPException(status_code=400, detail="phonenumbers is not valid")
    return telephone



from functools import lru_cache
from pydantic import BaseSettings, BaseModel, RedisDsn, PostgresDsn, PyObject
from jwtserver.internal.SMSC import SMSModel


class TokenModel(BaseModel):
    base_sol: str = '1234567890987654321'
    secret_key: str = None
    algorithm: str = 'HS256'
    access_expire_time: int = 90  # minutes
    refresh_expire_time: int = 10800  # minutes


class PostgresConfig(BaseModel):
    # pg_dsn: PostgresDsn = 'postgresql+asyncpg://jwtserver:jwtserver-password@localhost:5433/jwtserver-tests'
    pg_dsn: PostgresDsn = None
    # pg_dsn_sync: PostgresDsn = 'postgresql://jwtserver:jwtserver-password@localhost:5433/jwtserver-tests'
    pg_dsn_sync: PostgresDsn = None


class RedisConfig(BaseModel):
    # redis_dsn: RedisDsn = 'redis://:@localhost:6379/1'
    redis_dsn: RedisDsn = None
    max_connections: int = 10


class RecaptchaModel(BaseModel):
    secret_key: str = 'uudud'
    score: float = 0.7
    for_tests: bool = False


class GoogleConfig(BaseModel):
    Recaptcha: RecaptchaModel = RecaptchaModel()


class ServerConfig(BaseModel):
    # domain: Set[str] = set()
    host: str = '0.0.0.0'
    port: int = 8000
    max_requests: int = 1000
    debug: bool = True


a = ServerConfig()

class Settings(BaseSettings):
    environment: str = 'production'
    server: ServerConfig = ServerConfig()
    token: TokenModel = TokenModel()
    postgres: PostgresConfig = PostgresConfig()
    redis: RedisConfig = RedisConfig()
    Google: GoogleConfig = GoogleConfig()
    sms: SMSModel = SMSModel()

    class Config:
        env_file = '.env'
        env_nested_delimiter = '__'
        # env_prefix = ''


@lru_cache()
def get_settings():
    return Settings()

# def get_config():
#     raise NotImplementedError

# @lru_cache
# class Settings:
#     def __init__(self, _config=None):
#         self.config = _config or self.load_config()
#
#     def __call__(self, *args, **kwargs) -> ConfigModel:
#         return self.config
#
#     @staticmethod
#     def load_config() -> ConfigModel:
#         """
#         Load default and user config. Merge configs (override default values) and typing.
#         :return: merge configs.
#         :rtype: Config.
#         """
#
#         # Load default.ini file and parsing.
#         default_ini = pkg_resources.open_text(jwtserver, 'default.ini')
#         default_cfg = configparser.ConfigParser()
#         default_cfg.read_file(default_ini)
#
#         # Load config.ini file (User config) and parsing.
#         user_ini = configparser.ConfigParser()
#         user_ini.read('config.ini')
#
#         # Merge dicts.
#         merged_dict = defaultdict(dict)
#         merged_dict.update(default_cfg)
#
#         for key, nested_dict in user_ini.items():
#             merged_dict[key].update(nested_dict)
#
#         return ConfigModel(**dict(merged_dict))
#
#     def get_config(self) -> ConfigModel:
#         return self.config
#
#     def get_tests_config(self) -> ConfigModel:
#         db_url = 'jwtserver:jwtserver-password@localhost:5433/jwtserver-tests'
#         self.config.redis.url = "redis://:@localhost:6380/1"
#         self.config.db.sync_url = f"postgresql://{db_url}"
#         self.config.db.async_url = f"postgresql+asyncpg://{db_url}"
#         return self.config



from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.asyncio import create_async_engine
from sqlalchemy.orm import sessionmaker
from jwtserver import settings

settings = settings.get_settings()
pg_dsn = settings.postgres.pg_dsn

async_engine = create_async_engine(pg_dsn)
AsyncSessionLocal = sessionmaker(async_engine, expire_on_commit=False, class_=AsyncSession)

Base = declarative_base()



import httpx
from typing import Literal
from loguru import logger
from fastapi import HTTPException, Body
from pydantic import BaseModel
from starlette import status


class RecaptchaConfig(BaseModel):
    secret_key: str
    score: float
    for_tests: bool


class Recaptcha:
    """Recaptcha v3
    :raises HTTPException:
    https://www.google.com/recaptcha/admin/create"""

    def __init__(self, _config, recaptcha_token, environment):
        self.config: RecaptchaConfig = _config
        self.environment = environment
        self.action_name = None
        self.success = False
        self.action_valid = False
        self.r_json = None
        self.recaptcha_token = recaptcha_token

    def set_action_name(self, name) -> 'Recaptcha':
        """Set google action name"""
        self.action_name = name
        return self

    async def greenlight(self) -> Literal[True]:
        """If the recaptcha is ok, then the light is green.
        our minimum checks
        :return: True or raise HTTPException
        """

        def bad_request(detail: str):
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=detail,
            )

        await self.check()
        if not self.r_json['success']:
            logger.critical("invalid reCAPTCHA token")
            bad_request('server error: invalid reCAPTCHA token for your site')

        if not self.r_json['action'] == self.action_name:
            bad_request('hm... u r hacker?')

        if self.r_json['score'] <= self.config.score:
            bad_request('hm... u r bot?')

        return True

    async def check(self) -> 'Recaptcha':
        """send post and save response json to self.r_json"""
        if self.environment == 'tests':
            self.r_json = {
                "success": self.recaptcha_token.split(":")[0],
                "action": self.recaptcha_token.split(":")[1],
                "score": float(self.recaptcha_token.split(":")[2]),
            }
            return self
        data = {
            'secret': self.config.secret_key,
            'response': self.recaptcha_token,
        }
        async with httpx.AsyncClient() as client:
            r: httpx.Response = await client.post(
                'https://www.google.com/recaptcha/api/siteverify', data=data)
            self.r_json = r.json()
        return self



import pathlib
import setuptools
import codecs
import os.path

here = pathlib.Path(__file__).parent.resolve()

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name='jwtserver',
    author='Darkdeal',
    version=get_version("src/jwtserver/__init__.py"),
    author_email='real@darkdeal.net',
    description='jwt authorization server',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='Apache2',
    url='https://github.com/darkdealnet/jwtserver',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: Apache Software License',
    ],
    packages=setuptools.find_packages(
        where='src',
        include=['jwtserver', 'jwtserver.*'],
        exclude=['jwtserver.tests']
    ),
    package_dir={'': 'src'},
    python_requires=">=3.10",
    install_requires=[
        'uvicorn==0.16.0',
        'loguru==0.5.3',
        'aioredis==2.0.0',
        'fastapi==0.70.1',
        'sqlalchemy==1.4.28',
        'sqlalchemy_utils==0.37.9',
        'asyncpg==0.25.0',
        'psycopg2-binary==2.9.2',
        'httpx==0.21.1',
        'phonenumbers==8.12.39',
        'python-jose==3.3.0',
        'passlib==1.7.4',
        'starlette~=0.16.0',
        'pydantic~=1.8.2',
    ],
    package_data={
        'jwtserver': ['default.ini'],
    },
)



from loguru import logger
from pydantic import BaseModel
from sqlalchemy.exc import NoResultFound
from sqlalchemy.future import select
from starlette import status

from jwtserver import crud
from jwtserver.api.v1.help_func.ParseToken import TokenProcessor
from jwtserver.Google.Recaptcha_v3 import Recaptcha
from jwtserver.app import app
from sqlalchemy.ext.asyncio import AsyncSession
from fastapi import Depends, Response, HTTPException, Body
from jwtserver.functions.secure import verify_password
from jwtserver.functions.session_db import async_db_session
from jwtserver.models import User
from jwtserver.functions.config import load_config

token_config = load_config().token

response_description = """
There will also be response set_cookie refresh_token.
"""


class LoginResponseModel(BaseModel):
    access_token: str
    token_type: str


@app.post("/api/v1/login/",
          response_model=LoginResponseModel,
          tags=["Authorization"],
          description="User authorization by login and password",
          response_description=response_description,
          )
async def login(
        response: Response,
        recaptcha: Recaptcha = Depends(Recaptcha),
        telephone: str = Body(...),
        password: str = Body(...)
):
    """
    More here https://jwtserver.darkdeal.net/en/api_v1/#login
    :param response: Fastapi response
    :param recaptcha: https://jwtserver.darkdeal.net/en/recaptcha_v3/
    :param telephone: international phone number format
    :param password: string
    :raises HTTPException: recaptcha_v3 raises
    :raises HTTPException: If there is no user
    :return LoginResponse
    """
    await recaptcha.set_action_name('LoginPage/LoginButton').greenlight()
    user = await crud.get_user(telephone)
    logger.debug(telephone, password)

    if not user:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Неправильный номер или пароль",
            headers={"WWW-Authenticate": "JSv1"},
        )

    if verify_password(password, user.password):
        token_processor = TokenProcessor()
        logger.debug(token_processor)
        access_token, refresh_token = token_processor.create_pair_tokens(user.uuid.hex)
        logger.debug(access_token, refresh_token)
        logger.debug(access_token, refresh_token)
        response.set_cookie(
            key='refresh_token',
            value=refresh_token,
            httponly=True,
            secure=True,
            max_age=token_config.refresh_expire_time * 60)

        logger.debug({"access_token": access_token, "token_type": "JSv1"})
        return {"access_token": access_token, "token_type": "JSv1"}

    else:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Неправильный номер или пароль",
            headers={"WWW-Authenticate": "Bearer"},
        )



from sqlalchemy.exc import NoResultFound

from jwtserver.database import AsyncSessionLocal
from jwtserver.functions.session_db import async_db_session
from jwtserver.models import User
from sqlalchemy.future import select


async def get_user(telephone: str):
    session = async_db_session()
    session.next()
    session.execute()
    async with AsyncSessionLocal() as session:
        stmt = select(User).where(User.telephone == telephone)
        result = await session.execute(stmt)
        try:
            return result.scalars().one()
        except NoResultFound:
            return None



from pydantic import BaseModel
from jwtserver.app import app
from fastapi import Response


class LogoutResponseModel(BaseModel):
    status: str


@app.get("/api/v1/logout/", response_model=LogoutResponseModel, tags=["Authorization"],
         description="Removes token from cookie")
async def logout(response: Response):
    """If the exit, delete the token and that's it
    :param response: fastapi.Response
    :return LogoutResponse
    """
    response.delete_cookie(key='refresh_token')
    return {"status": 'logout'}



import logging
import sys

from fastapi import FastAPI
from loguru import logger
from starlette.middleware.cors import CORSMiddleware
from jwtserver import __version__
from jwtserver.api.v1.routers.reg import router as v1_router

__all__ = ['app', 'create_app']

origins = [
    "http://localhost",
    "http://localhost:8080",
    "http://localhost:8000",
    "http://localhost:5000",
    "http://localhost:3000",
]
description = """[Full JWT Server docs](https://jwtserver.darkdeal.net)"""
tags_metadata = [
    {
        "name": "Authorization",
        "description": "authorization",
    },
    {
        "name": "Registration",
        "description": "registration",
        "externalDocs": {
            "description": "Items external docs",
            "url": "https://jwtserver.darkdeal.net/en/api_v1/",
        },
    },
]


def create_app(_title='JWT server', lvl_logging='INFO') -> FastAPI:
    _app = FastAPI(
        title=_title,
        description=description,
        version=__version__,
        license_info={
            "name": "Apache 2.0",
            "url": "https://www.apache.org/licenses/LICENSE-2.0.html",
        },
        openapi_tags=tags_metadata,
    )
    _app.add_middleware(
        CORSMiddleware,
        allow_origins=origins,
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )

    _app.include_router(v1_router)
    # _app.dependency_overrides[get_settings] = get_settings
    # _app.dependency_overrides = {
    #     get_settings: lambda: _config,
    # }
    _app.debug = True

    # def enable_logger(sink=sys.stderr, level='DEBUG'):
    #     logging.basicConfig(level=logging.DEBUG)
    #     logger.configure(handlers=[{'sink': sink, 'level': level}])
    #     logger.enable('aria2p')
    #
    # enable_logger(level=lvl_logging)
    return _app


app = create_app()



# from jwtserver.api.v1.routers.reg.phone_status import phone_status
# from jwtserver.api.v1.routers.reg.send_code import call_code
# from jwtserver.api.v1.routers.reg.check_code import check_code
# from jwtserver.api.v1.routers.reg.signup import reg_user
# from jwtserver.api.v1.routers.auth.update_token import update_token
# from jwtserver.api.v1.routers.auth.logout import logout
# from jwtserver.api.v1.routers.auth.login import login
#



"""JWTServer is a lightweight and fast JWT microservice."""
__version__ = '0.0.17'

from jwtserver.app import app as app
from jwtserver.app import create_app as create_app
from jwtserver.server import dev as dev
